/************************************************************************/
/* Fichero: bufferfijo.c                                                */
/* Descripc�n: fichero que implementa un buffer de playout fijo. Lee   */
/* de un fichero de entrada el retraso en milisegundos escribiendo en   */
/* un fichero un 1 si el paquete se recibe a tiempo o un 0 si no        */
/* Llamada: bufferfijo buffer_delay fich_retrasos fich_transmitidos     */
/************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "algoritmos.h"

int main( int argc, char *argv[] )
{
   FILE    *Ifp, *Ofp, *dibujofp;            /* I/O File pointers */
   float BufferDelay;    // Tiempo de estancia en el buffer de playout
   float PlayoutDelay;   // Tiempo desde que se transmite un paquete hasta que se reproduce
   float delay;          // Retraso actual leido del fichero
   float silencio;       // Duracion del periodo de silencio
   int secuencia;        // Numero de secuencia del paquete recibido

   silencio = 0;

   if (argc != 5 ) {
      printf("Llamada: %s buffer_delay(ms) fich_retrasos fich_transmitidos fich_dibujo\n", argv[0]);
      exit(1);
   }
   
   Ifp = fopen( argv[2], "r") ;
   if ( Ifp == NULL ) {
      fprintf(stderr, "Invalid input file name: %s\n", argv[2] ) ;
      exit(1) ;
   }

   Ofp = fopen( argv[3], "w") ;
   if ( Ofp == NULL ) {
      fprintf(stderr, "Can't open output file: %s\n", argv[3] ) ;
      exit(1) ;
   } 

   dibujofp = fopen( argv[4], "w") ;
   if ( dibujofp == NULL ) {
      fprintf(stderr, "Can't open output file: %s\n", argv[4] ) ;
      exit(1) ;
   } 
 
   BufferDelay = atof(argv[1]);

   delay = Lee_retraso(&silencio, &secuencia, Ifp, Ofp, dibujofp);
   silencio = 0;

   PlayoutDelay = delay + BufferDelay;  // Se toma como referencia el retraso del primer paquete
   fprintf(dibujofp, "%d %0.2f\n", secuencia, PlayoutDelay);
   fputs("1 ", Ofp);    // El primer paquete se recibe a tiempo

   delay = Lee_retraso(&silencio, &secuencia, Ifp, Ofp, dibujofp);
   // se lee el fichero completo
   //feof: devuelve un valor distinto de cero si ha llegado al final del archivo 
   //ferror: devuelve un valor distinto de cero si se ha producido algun error
   while(!feof(Ifp) && !ferror(Ifp) && !ferror(Ofp)){

      Playout_buffer(&silencio, &delay, &PlayoutDelay, BufferDelay, secuencia, dibujofp);

      if(delay <= PlayoutDelay){   // El paquete se recibe a tiempo
         fputs("1 ", Ofp);
      }
      else {                       // El paquete no llega a transmitirse
         fputs("0 ", Ofp);
      }
      delay = Lee_retraso(&silencio, &secuencia, Ifp, Ofp, dibujofp);
   }

   // Se cierran los ficheros
   if(Ifp) { 
      (void)fclose(Ifp);
   }
   if(Ofp) { 
      (void)fclose(Ofp); 
   }
   if(dibujofp) { 
      (void)fclose(Ofp); 
   }
      
   return 0 ;
}

float Lee_retraso(float *silencio, int *secuencia, FILE *Ifp, FILE *Ofp, FILE
*playoutfp, FILE *bufferdelayfp){
   char cadena[20];
   float delay;
   fscanf(Ifp, "%s", cadena);
   while(cadena[0] == '#' && !feof(Ifp) && !ferror(Ifp) && ! ferror(Ofp)){
      if(!strcmp(cadena, "#Silencio")){ // Es un periodo de silencio
           fscanf(Ifp, "%f", silencio);
      }
      else{ // Es un paquete perdido
          *secuencia = atoi(&cadena[1]);
           fscanf(Ifp, "%s", cadena);
         fprintf(playoutfp, "#%d %s\n", *secuencia, cadena);
         fprintf(bufferdelayfp, "#%d %s\n", *secuencia, cadena);
         fputs("0 ", Ofp);
     }
     fscanf(Ifp, "%s", cadena);
  }
  *secuencia = atoi(cadena);
  fscanf(Ifp, "%f", &delay);
  return delay;
}


int Playout_buffer(float *silencio, float *delay, float *PlayoutDelay, float \
                BufferDelay, int secuencia, FILE *playoutfp){
   float antPlayoutDelay;
   if(*silencio != 0){
      antPlayoutDelay = *PlayoutDelay;
      *PlayoutDelay = *delay + BufferDelay;     // Se ajusta con el primer
                                                // paquete de la rafaga
      if(*PlayoutDelay < antPlayoutDelay - *silencio){ // Se solapa con el
                                               // ultimo paquete de la rafaga
        *PlayoutDelay = antPlayoutDelay - *silencio;
      }
      fprintf(playoutfp, "%d %0.4f\n", secuencia, *PlayoutDelay);
      *silencio = 0;
   }
   else{
      fprintf(playoutfp, "%d %0.4f\n", secuencia, *PlayoutDelay);
     // En la rafaga se mantiene el mismo valor
   }
   return 0;
}




