/************************************************************************/
/* Fichero: desencapsulado.c                                            */
/* Descripcon: fichero que desencapsula las tramas contenidas en los   */
/* paquetes IP. Lee de los ficheros de entrada el tipo de trama obtenida*/
/* inicialmente en el codificador y los paquetes que se han perdido.    */
/* Escribe en un fichero de salida las tramas que finalmente llegan al  */
/* decodificador: 0 perdidas, 1 transmitidas                            */
/* Llamada: desencapsulado codificador fich_paquetes                    */
/*          fich_paquetes_transmitidos fich_tramas_transmitidas         */
/************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define T_TRAMA_G723 30   //Codificador g723: tramas de 30ms
#define T_TRAMA_G729 10   //Codificador g729: tramas de 10ms

#define PERDIDO '#'
#define TRANSMITIDO '1'
#define NO_TX '0'

int main( int argc, char *argv[] )
{
   FILE    *Ifp_paquetes, *Ifp_tx, *Ofp ;            /* I/O File pointers */
   char tx[30];          // Indica si el paquete llega a transmitirse
   float tiempo;
   char cadena[30];
   int escritas;
   int no_tx;
   int t_trama;          // Duracion de una trama
   int pos;
   int fin;
   int secuencia;
   int leer;

   if (argc !=5) {
      printf("Llamada: %s codificador fich_paquetes fich_paquetes_tx fich_tramas_tx\n", argv[0]);
      exit(1);
   }

  // Inicializo variables
   no_tx = 0;
   escritas = 0;
   pos = 0;
   fin = 0;
   leer = 1;

   Ifp_paquetes = fopen( argv[2], "r") ;
   if ( Ifp_paquetes == NULL ) {
      fprintf(stderr, "Nombre de entrada no valido: %s\n", argv[2] ) ;
      exit(1) ;
   }

   Ifp_tx = fopen( argv[3], "r") ;
   if ( Ifp_tx == NULL ) {
      fprintf(stderr, "Nombre de entrada no valido: %s\n", argv[3] ) ;
      exit(1) ;
   }

   Ofp = fopen( argv[4], "w") ;
   if ( Ofp == NULL ) {
      fprintf(stderr, "No se puede abrir fichero de salida: %s\n", argv[4] ) ;
      exit(1) ;
   }

   if( strcmp(argv[1], "g723")== 0){
      t_trama = T_TRAMA_G723;
   }
   else if( strcmp(argv[1], "g729")== 0){
      t_trama = T_TRAMA_G729;
   } 
   else{
      fprintf(stderr, "Codificador no valido. Opciones: g723 o g729\n");
      exit(1);
   }

   // Se lee el numero de tramas que contiene cada paquete
   fscanf(Ifp_paquetes, "%s", cadena);
   while(!strcmp("#Silencio", cadena) && !feof(Ifp_paquetes) && !ferror(Ifp_paquetes)){
       fscanf(Ifp_paquetes, "%f", &tiempo);
       no_tx = tiempo/t_trama;
       while(no_tx > 0){
          fprintf(Ofp, "1 ");
	   no_tx--;
	   escritas++;
       }
       fscanf(Ifp_paquetes, "%s", cadena);
   }
   secuencia = atoi(cadena);
   fscanf(Ifp_paquetes, "%s %s %s %s %s %s %s %s %s %s %s", cadena, cadena, cadena, cadena, cadena, cadena, cadena, cadena, cadena, cadena, cadena);
  
   fscanf(Ifp_tx, "%s", tx);
   while( !feof(Ifp_tx) && !ferror(Ifp_tx) && !feof(Ifp_paquetes) && !ferror(Ifp_paquetes) && !ferror(Ofp)){
      pos = 0;
      if(tx[0] == PERDIDO){
         fscanf(Ifp_tx, "%s", tx);
         leer = 0;
         if(!strcmp("Perdido_red", tx) || !strcmp("Perdido_token", tx)){  //Se pierde el paquete completo
	    while(cadena[pos] != '\0'){
	       pos++;
	       fprintf(Ofp, "0 ");
               escritas++;
	    }
	 }
      }
      while(cadena[pos] != '\0'){  //Caso en que se transmita o un trozo se pierda en el buffer
	 pos++;
         if(cadena[pos] != NO_TX){
            if(no_tx > 0){
               if(leer != 0){
	          fscanf(Ifp_tx, "%s", tx);
	       }
               fscanf(Ifp_tx, "%s ", tx);
               leer = 1;
	       no_tx = 0;
	    }
            if(!strncmp("Perdido", tx, 7)){ //El paquete se ha perdido
               escritas++;
	       fprintf(Ofp, "0 ");
	    }
            else{  //El paquete se ha transmitido
	       escritas++;
	       fprintf(Ofp, "1 ");
	    }
	 }
         else{  // Hay silencio 
	    no_tx++;
            fprintf(Ofp, "1 ");
	    escritas++;
	 }
      }
      //Se lee el estado de transmision de cada paquete
      if(leer != 0){
         fscanf(Ifp_tx, "%s", tx);
      }
      fscanf(Ifp_tx, "%s", tx);
      leer = 1;

      // Se lee el numero de tramas que contiene cada paquete
      fscanf(Ifp_paquetes, "%s", cadena);
      while(!strcmp("#Silencio", cadena) && !feof(Ifp_paquetes) && !ferror(Ifp_paquetes)){
         fscanf(Ifp_paquetes, "%f", &tiempo);
         no_tx = tiempo/t_trama;
         while(no_tx > 0){
            fprintf(Ofp, "1 ");
	     no_tx--;
	     escritas++;
         }
         fscanf(Ifp_paquetes, "%s", cadena);
      }
      secuencia = atoi(cadena);
      fscanf(Ifp_paquetes,"%s %s %s %s %s %s %s %s %s %s %s", cadena, cadena, cadena, cadena, cadena, cadena, cadena, cadena, cadena, cadena, cadena);  
   }
   //fprintf(stdout, "Tramas desencapsuladas: %d\n", escritas);
   // Se cierran los ficheros
   if(Ifp_tx) { 
      (void)fclose(Ifp_tx);
   }
   if(Ifp_paquetes) { 
      (void)fclose(Ifp_paquetes);
   }
   if(Ofp) { 
      (void)fclose(Ofp); 
   }
   return 0 ;
}





