/************************************************************************/
/* Fichero: encapsulado.c                                               */
/* Descripcon: fichero que encapsula las tramas en paquetes IP. Lee de */
/* un fichero de entrada el tipo de trama obtenida en el codificador y  */
/* espera a formar un paquete. Escribe en un fichero de salida el       */
/* instante de formacion de cada paquete y las tramas que contiene. En  */
/* otro fichero escribe el retraso de formacion del paquete.            */
/* Se pueden seleccionar dos metodos de encapsulado: segun la RFC 3551  */
/* o el metodo multisid.                                                */
/* Llamada: encapsulado tasa_empaquetado fich_tipotramas fich_paquetes  */
/*          codificador tipo_encapsulado (RFC o multisid)               */
/************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include "encapsulado.h"

int main( int argc, char *argv[] )
{
   FILE    *Ifp, *Ofp, *retrasofp ;   /* I/O File pointers */

   int tasa_empaquetado; // Numero de tramas en cada paquete
   int t_trama;          // Tiempo de trama
   float look_ahead;     // Tiempo para obtener informacion de la siguiente trama

   if (argc !=7) {
      printf("Llamada: %s tasa_empaquetado fich_tipotramas fich_paquetes \
fich_retraso codificador tipo_encapsulado\n", argv[0]);
      printf("Codificador: g723 - g729\n");
      printf("Tipo_encapsulado: RFC o multisid\n");
      exit(1);
   }

   Ifp = fopen( argv[2], "r") ;
   if ( Ifp == NULL ) {
      fprintf(stderr, "Nombre del fichero de entrada no valido: %s\n", argv[2] ) ;
      exit(1) ;
   }

   Ofp = fopen( argv[3], "w") ;
   if ( Ofp == NULL ) {
      fprintf(stderr, "No se puede abrir el fichero de salida: %s\n", argv[3] ) ;
      exit(1) ;
   }

   retrasofp = fopen( argv[4], "w") ;
   if ( retrasofp == NULL ) {
      fprintf(stderr, "No se puede abrir el fichero de salida: %s\n", argv[4] ) ;
      exit(1) ;
   }

   if( strcmp(argv[5], "g723")== 0){
      t_trama = T_TRAMA_G723;
      look_ahead =  LOOK_AHEAD_G723;
   }
   else if( strcmp(argv[5], "g729")== 0){
      t_trama = T_TRAMA_G729;
      look_ahead =  LOOK_AHEAD_G729;
   }
   else{
      fprintf(stderr, "Codificador no valido. Opciones: g723 o g729\n");
      exit(1);
   }

   tasa_empaquetado = atoi(argv[1]);

   if( !strcmp(argv[6], "RFC") ){
      Encapsulado_RFC(tasa_empaquetado, t_trama, look_ahead, Ifp, Ofp, retrasofp);
   }
   else if( !strcmp(argv[6], "multisid") ){
      Encapsulado_Multisid(tasa_empaquetado, t_trama, look_ahead, Ifp, Ofp, retrasofp);
   }
   else{
      fprintf(stderr, "Tipo de encapsulado no valido: RFC o multisid\n");
   }

   // Se cierran los ficheros
   if(Ifp) { (void)fclose(Ifp);  }
   if(Ofp) { (void)fclose(Ofp);  }
   if(retrasofp) { (void)fclose(retrasofp);  }
    
   return 0 ;
}

