/************************************************************************/
/* Fichero: fastexponencial.c                                           */
/* Descripcon: fichero que implementa un buffer de playout exponencial */
/* media. Lee de un fichero de entrada el retraso en milisegundos       */
/* escribiendo en un fichero un 1 si la trama se recibe a tiempo o un 0 */
/* si no.                                                               */
/* Llamada: exponencial fich_retrasos fich_transmitidos                 */
/************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>  //para usar la funcion fabs (en include-mingw32)

#include "algoritmos.h"

int main( int argc, char *argv[] )
{
   FILE    *Ifp, *Ofp, *dibujofp;            /* I/O File pointers */

   double PlayoutDelay;      // Valor que se va calculando con el algoritmo exponencial
   double ref_PlayoutDelay; // Valor que se ajusta al principio de cada rafaga de voz
   double delay;            // Retraso del paquete leido del fichero
   float silencio;         // Duracion del periodo de silencio
            
   double media;
   double varianza;
   int secuencia;
   FILE *borrar = fopen("pfast.txt", "w");
   silencio = 0;

   if (argc != 4 ) {
      printf("Llamada: %s fich_retrasos fich_transmitidos fich_dibujo \n", argv[0]);
      exit(1);
   }
   
   Ifp = fopen( argv[1], "r") ;
   if ( Ifp == NULL ) {
      fprintf(stderr, "Invalid input file name: %s\n", argv[1] ) ;
      exit(1) ;
   }

   Ofp = fopen( argv[2], "w") ;
   if ( Ofp == NULL ) {
      fprintf(stderr, "Can't open output file: %s\n", argv[2] ) ;
      exit(1) ;
   }

   dibujofp = fopen( argv[3], "w") ;
   if ( dibujofp == NULL ) {
      fprintf(stderr, "Can't open output file: %s\n", argv[3] ) ;
      exit(1) ;
   }
   
   delay = Lee_retraso(&silencio, &secuencia, Ifp, Ofp, dibujofp);
   silencio = 0;

   media = delay;
   varianza = 0;

   PlayoutDelay = media + (4 * varianza);
   ref_PlayoutDelay = PlayoutDelay;
   fprintf(dibujofp, "%d %0.2f\n", secuencia, PlayoutDelay); 

   fputs("1 ", Ofp);    // Escribe un 1 en el fichero (el primer paquete se recibe a tiempo)

   delay = Lee_retraso(&silencio, &secuencia, Ifp, Ofp, dibujofp);
   
   // se lee el fichero completo
   //feof: devuelve un valor distinto de cero si ha llegado al final del archivo 
   //ferror: devuelve un valor distinto de cero si se ha producido algun error
   while(!feof(Ifp) && !ferror(Ifp) && !ferror(Ofp)){

      Playout_fastexp(&silencio, &delay, &PlayoutDelay, &ref_PlayoutDelay, secuencia, &media, &varianza, dibujofp);

      if (delay < ref_PlayoutDelay){   // el paquete se recibe a tiempo
         fputs("1 ", Ofp);
      }
      else {                       // el paquete no llega a transmitirse
         fputs("0 ", Ofp);
      }
      fprintf(borrar, "%d %f\n", secuencia, PlayoutDelay);
      delay = Lee_retraso(&silencio, &secuencia, Ifp, Ofp, dibujofp);
   }
   // Se cierran los ficheros
   if(Ifp) { 
      (void)fclose(Ifp);
   }
   if(Ofp) { 
      (void)fclose(Ofp); 
   }
   if(dibujofp) { 
      (void)fclose(Ofp); 
   }
      
   return 0 ;
}



