/************************************************************************/
/* Fichero: fencapsulado.c                                              */
/* Descripcon: fichero que implementa las funciones usadas por         */
/* encapsulado.c                                                        */
/************************************************************************/

#include <stdio.h>

#include "encapsulado.h"

/* Funcion Encapsulado_RFC */
/* Funcion que implementa el empaquetado de tramas segun la RFC 3551 */
int Encapsulado_RFC (int tasa_empaquetado, int t_trama, float look_ahead, \
		     FILE *Ifp, FILE *Ofp, FILE *retrasofp){

   int tipo;             // Tipo de trama leida del fichero de entrada
   int activa;           // Contador de tramas activas
   int sid;              // Contador de tramas sid
   int no_tx;            // Contador de tramas no transmitidas
   float tiempo;         // Instante de tiempo actual
   int paquetes;         // Numero de secuencia del paquete
   char mascara[15];     // Secuencia de los tipos de trama leidos
   int trama;            // Numero de tramas leidas en cada paquete
   int silencio;         // Contador de la duracion de silencio entre rafagas
   int tx;               // Indicador de transmision del paquete
   int i;

   // Inicializacion de variables
   activa = 0;
   sid = 0;
   no_tx = 0;
   tiempo = 0;
   paquetes = 1;
   trama = 0;
   silencio = 0;
   tx = 0;

   for(i = 0; i < 15; i++){
      mascara[i] = ' ';
   }

   tipo = fgetc(Ifp);
   tiempo = tiempo + t_trama;  // Ha transcurrido el tiempo correspondiente a una trama

   while(!feof(Ifp) && !ferror(Ifp) && !ferror(Ofp)){

      if(tipo == ACTIVA){           // Trama activa
	 activa++;
         mascara[trama] = ACTIVA;
         trama++;
	 if(silencio > 0){      // Finaliza un periodo de silencio
	    fprintf(Ofp, "#Silencio %d\n", silencio);
            fprintf(retrasofp, "#Silencio %d\n", silencio);
	    silencio = 0;
	 }
      } 
      else if(tipo == SID){      // Trama SID
	 sid++; 
         mascara[trama] = SID;
         trama++;                 
	 if(t_trama == T_TRAMA_G729){ // Caso del g729: solo puede ir una trama sid por paquete
	    tx = 1;                   // Condicion para la transmision del paquete
	 }
	 if(silencio > 0){       // Finaliza un periodo de silencio
	    fprintf(Ofp, "#Silencio %d\n", silencio);
            fprintf(retrasofp, "#Silencio %d\n", silencio);
	    silencio = 0;
	 }
      }
      else if(tipo == NO_TX){ // Trama no transmitida: solo puede ir despues de trama sid o no_tx
	 if(sid > 0){   // El paquete contiene trama SID en el caso del g723 (el g729 ya ha transmitido)
	    mascara[trama] = '\0';
            fprintf(Ofp, "%d  %0.1f  Activas: %d  SID: %d  No_tx: %d  Cabecera: %d  Tramas: %s\n",\
		    paquetes, tiempo+look_ahead, activa, sid, no_tx, CABECERA, mascara);
            fprintf(retrasofp, "%d %0.2f\n", paquetes, (activa+sid) * t_trama + look_ahead);
            trama = 0;
            paquetes++;
	    activa = 0;
	    sid = 0;
	    tx = 0;        
	 }	       // No ha comenzado la formacion de un paquete: empieza un periodo de silencio
         silencio = silencio + t_trama;  // Se calcula el tiempo entre rafagas de voz
      }
      else{
         fprintf(stderr, "Tipo de trama leida no valida") ;
         exit(1) ;
      }

      if((activa + sid) == tasa_empaquetado || tx == 1){  // Se ha formado un paquete
	 mascara[trama] = '\0';
         fprintf(Ofp, "%d  %0.1f  Activas: %d  SID: %d  No_tx: %d  Cabecera: %d  Tramas: %s\n",\
		 paquetes, tiempo+look_ahead, activa, sid, no_tx, CABECERA, mascara);
         fprintf(retrasofp, "%d %0.2f\n", paquetes, (activa+sid) * t_trama + look_ahead);

         trama = 0;
         paquetes++;      
         activa = 0;
         sid = 0;
	 tx = 0;
      }	 	  

      tipo = fgetc(Ifp);
      tipo = fgetc(Ifp);
      tiempo = tiempo + t_trama;  // Transcurre el tiempo de una trama
   }

   //Final del fichero y puede haber un paquete en formacion que no se ha transmitido
   if((activa + sid) < tasa_empaquetado && (activa + sid) != 0 ){
      mascara[trama] = '\0';
      fprintf(Ofp, "%d  %0.1f  Activas: %d  SID: %d  No_tx: %d  Cabecera: %d  Tramas: %s\n",\
	      paquetes, tiempo+look_ahead, activa, sid, no_tx, CABECERA, mascara);
      fprintf(retrasofp, "%d %0.2f\n", paquetes, (activa+sid)*t_trama + look_ahead);
      paquetes++;
   }
   //fprintf(stdout, "%d paquetes formados\n", paquetes-1);
   return 0;
}



/* Funcion Encapsulado_Multisid */
/* Funcion que implementa el empaquetado de tramas en el que se permite tener mas de */
/* una trama sid por paquete */
int Encapsulado_Multisid(int tasa_empaquetado, int t_trama, float look_ahead, FILE *Ifp,\
			 FILE *Ofp, FILE *retrasofp){

   int tipo;             // Tipo de trama leida del fichero de entrada
   int activa;           // Contador de tramas activas
   int sid;              // Contador de tramas sid
   int no_tx;            // Contador de tramas no transmitidas
   float tiempo;         // Instante de tiempo actual
   int paquetes;         // Numero de secuencia del paquete
   int cabecera;         // Tamano en bytes de la cabecera del paquete
   char mascara[15];     // Secuencia de los tipos de trama leidos
   int trama;            // Numero de tramas leidas en cada paquete
   int silencio;         // Contador de la duracion de silencio entre rafagas
   int i;
   int retraso;

   // Inicializacion de variables
   activa = 0;
   sid = 0;
   no_tx = 0;
   tiempo = 0;
   paquetes = 1;
   trama = 0;
   silencio = 0;
   retraso = 0;

   for(i = 0; i < 15; i++){
      mascara[i] = ' ';
   }
   tipo = fgetc(Ifp);
   tiempo = tiempo + t_trama;   //Transcurre el tiempo de una trama

   while(!feof(Ifp) && !ferror(Ifp) && !ferror(Ofp)){
      
      if(tipo == ACTIVA){        // Trama activa
	 activa++;
         mascara[trama] = ACTIVA;
         trama++;
	 if(silencio > 0){      // Finaliza un periodo de silencio
	    fprintf(Ofp, "#Silencio %d\n", silencio);
            fprintf(retrasofp, "#Silencio %d\n", silencio);
	    silencio = 0;
	 }
      }  
      else if(tipo == SID){      // Trama SID
	 sid++;
         mascara[trama] = SID;
         trama = trama++;
	 if(silencio > 0){      // Finaliza un periodo de silencio
	    fprintf(Ofp, "#Silencio %d\n", silencio);
            fprintf(retrasofp, "#Silencio %d\n", silencio);
	    silencio = 0;
	 }
      }
      else if(tipo == NO_TX){    // Trama no transmitida
	 if((activa > 0) || (sid > 0)){   //Solo empaqueta si hay alguna trama en el paquete en formacion
                                          //Sino no haria nada. Lo unico es q ha transcurrido tiempo
	    no_tx++;
            mascara[trama] = NO_TX;
            trama = trama++;
	 }
	 else{
	    silencio = silencio + t_trama;  
	 }
      }
      else{
         fprintf(stderr, "Tipo de trama leida no valida") ;
         exit(1) ;
      }

      if((activa + sid + no_tx) == tasa_empaquetado){  // Se ha formado un paquete
         mascara[trama] = '\0';
         while(mascara[trama-1] == NO_TX){
	    retraso = retraso + t_trama;
            trama--;
            mascara[trama] = '\0';
            no_tx--;   // Esta trama no va a formar parte del paquete. Es un periodo de silencio
            silencio = silencio + t_trama;
	 }
	 cabecera = Tamano_Cabecera (t_trama, sid, no_tx);
         fprintf(Ofp, "%d  %0.1f  Activas: %d  SID: %d  No_tx: %d  Cabecera: %d  Tramas: %s\n", \
		 paquetes, tiempo+look_ahead, activa, sid, no_tx, cabecera, mascara);
         fprintf(retrasofp, "%d %0.2f\n", paquetes, (activa+sid+no_tx) * t_trama + look_ahead + retraso);         

         paquetes++;
         activa = 0;
         sid = 0;
         no_tx = 0;
	 trama = 0;
         retraso = 0;
         for(i = 0; i < 15; i++){
            mascara[i] = ' ';
         }
      }
      tipo = fgetc(Ifp);
      tipo = fgetc(Ifp);

      tiempo = tiempo + t_trama;  //Transcurre el tiempo de una trama
   }

   //Puede ser que llegue al final del fichero y un paquete estaba en formacion
   if((activa + sid + no_tx) < tasa_empaquetado && (activa + sid + no_tx) != 0 ){
      mascara[trama] = '\0';
      while(mascara[trama-1] == NO_TX){
	 retraso = retraso + t_trama;
         trama--;
         mascara[trama] = '\0';
         no_tx--;   // Esta trama no va a formar parte del paquete. Es un periodo de silencio
         silencio = silencio + t_trama;
      }
      cabecera = Tamano_Cabecera (t_trama, sid, no_tx);
      fprintf(Ofp, "%d  %0.1f  Activas: %d  SID: %d  No_tx: %d  Cabecera: %d  Tramas: %s\n",\
	      paquetes, tiempo+look_ahead, activa, sid, no_tx, cabecera, mascara);
      fprintf(retrasofp, "%d %0.2f\n", paquetes, (activa+sid+no_tx) * t_trama + look_ahead + retraso);
      paquetes++;
   }  
   
   //fprintf(stdout, "%d paquetes formados\n", paquetes-1);

   return 0;
}


int Tamano_Cabecera (int t_trama, int sid, int no_tx){

   int cabecera;

   if(t_trama == T_TRAMA_G723){
      if(no_tx == 0){         // Si no hay tramas no_tx no se mejora
	 cabecera = CABECERA; // El g723 podia llevar mas de una trama sid
      }
      else{
	 if(sid > 1){     // Solo cambia el tamano si hay mas de una trama SID
	    cabecera = CABECERA + sid;
	 } 
         else{
	    cabecera = CABECERA;
	 }
      }
   }
   if(t_trama == T_TRAMA_G729){
      if(sid > 1){
	 cabecera = CABECERA + sid;
      }
      else{
	 cabecera = CABECERA;
      }
   }

   return cabecera;
}





