/************************************************************************/
/* Fichero: minimo.c                                                    */
/* Descripcon: fichero que implementa un buffer de playout que recoge  */
/* el minimo valor del retraso en una rafaga y lo utiliza para calcular */
/* el playout delay de la siguiente rafaga                              */
/* Llamada: minimo fich_retrasos fich_transmitidos dibujo               */
/************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "algoritmos.h"

int main( int argc, char *argv[] )
{
   FILE    *Ifp, *Ofp, *dibujofp ;            /* I/O File pointers */
   NODO *ldelay;
   float PlayoutDelay;      // Valor que se va calculando con el algoritmo exponencial
   float ref_PlayoutDelay; // Valor que se ajusta al principio de cada rafaga de voz
   float delay;            // Retraso del paquete leido del fichero
   float silencio;         // Duracion del periodo de silencio
   int secuencia;          // Numero de secuencia del paquete recibido
   float media;
   float varianza;
   FILE *borrar = fopen("pmin.txt", "w");
   silencio = 0;       // Duracion del periodo de silencio
   ldelay = NULL;

   if (argc != 4 ) {
      printf("Llamada: %s  fich_retrasos fich_transmitidos fich_dibujo\n", argv[0]);
      exit(1);
   }
   
   Ifp = fopen( argv[1], "r") ;
   if ( Ifp == NULL ) {
      fprintf(stderr, "Invalid input file name: %s\n", argv[1] ) ;
      exit(1) ;
   }

   Ofp = fopen( argv[2], "w") ;
   if ( Ofp == NULL ) {
      fprintf(stderr, "Can't open output file: %s\n", argv[2] ) ;
      exit(1) ;
   } 
   
   dibujofp = fopen( argv[3], "w") ;
   if ( dibujofp == NULL ) {
      fprintf(stderr, "Can't open output file: %s\n", argv[3] ) ;
      exit(1) ;
   }

   delay = Lee_retraso(&silencio, &secuencia, Ifp, Ofp, dibujofp);
   silencio = 0;

   Enlaza_orden(&ldelay, delay);  //Primer nodo de la cola

   media = delay;
   varianza = 0;

   PlayoutDelay = media + (4 * varianza);
   ref_PlayoutDelay = PlayoutDelay;
   fprintf(dibujofp, "%d %0.2f\n", secuencia, PlayoutDelay);
   fputs("1 ", Ofp);    // Escribe un 1 en el fichero (el primer paquete se recibe a tiempo)
   delay = Lee_retraso(&silencio, &secuencia, Ifp, Ofp, dibujofp);

   // se lee el fichero completo
   //feof: devuelve un valor distinto de cero si ha llegado al final del archivo 
   //ferror: devuelve un valor distinto de cero si se ha producido algun error
   while(!feof(Ifp) && !ferror(Ifp) && !ferror(Ofp)){
      Enlaza_orden(&ldelay, delay);
      Playout_minimo(&silencio, &delay, &PlayoutDelay, &ref_PlayoutDelay, \
		     secuencia, &media, &varianza, &ldelay, dibujofp);
	 
      if (delay < ref_PlayoutDelay){   // el paquete se recibe a tiempo
         fputs("1 ", Ofp);
      }
      else {                       // el paquete no llega a transmitirse
         fputs("0 ", Ofp);
      }

      fprintf(borrar, "%d %f\n", secuencia, PlayoutDelay);
      delay = Lee_retraso(&silencio, &secuencia, Ifp, Ofp, dibujofp);
   }

   if(Ifp) { 
      (void)fclose(Ifp);
   }
   if(Ofp) { 
      (void)fclose(Ofp); 
   }
   if(dibujofp) { 
      (void)fclose(Ofp); 
   }

   return 1;

}
