/************************************************************************/
/* Fichero: spike.c                                                     */
/* Descripc�n: fichero que implementa un buffer de playout que se      */
/* adapta a los impulsos de los retrasos. Lee de un fichero de entrada  */
/* el retraso en milisegundos escribiendo en un fichero un 1 si la      */
/* trama se recibe a tiempo o un 0 si no                                */
/* Llamada: spike fich_retrasos fich_transmitidos                       */
/************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>  //para usar la funcion fabs (en include-mingw32)

#include "algoritmos.h"

int main( int argc, char *argv[] )
{
   FILE    *Ifp, *Ofp, *dibujofp;            /* I/O File pointers */
   float PlayoutDelay;      // Valor que se va calculando con el algoritmo exponencial
   float ref_PlayoutDelay; // Valor que se ajusta al principio de cada rafaga de voz
   float delay;            // Retraso del paquete leido del fichero
   float silencio;         // Duracion del periodo de silencio       
   NODO *ldelay;
   NODO *nodo;
   float media;
   float varianza;
   float var;
   int mode;
   int secuencia;
   FILE *borrar = fopen("pspike.c", "w");
   silencio = 0;
   mode = NORMAL;
   var = 0;

   if (argc != 4 ) {
      printf("Llamada: %s fich_retrasos fich_transmitidos fich_dibujo \n", argv[0]);
      exit(1);
   }
   
   Ifp = fopen( argv[1], "r") ;
   if ( Ifp == NULL ) {
      fprintf(stderr, "Invalid input file name: %s\n", argv[1] ) ;
      exit(1) ;
   }

   Ofp = fopen( argv[2], "w") ;
   if ( Ofp == NULL ) {
      fprintf(stderr, "Can't open output file: %s\n", argv[2] ) ;
      exit(1) ;
   }

   dibujofp = fopen( argv[3], "w") ;
   if ( dibujofp == NULL ) {
      fprintf(stderr, "Can't open output file: %s\n", argv[3] ) ;
      exit(1) ;
   }

   delay = Lee_retraso(&silencio, &secuencia, Ifp, Ofp, dibujofp);
   silencio = 0;

   media = delay;
   varianza = 0;
   ldelay = CreaNodo(delay, 0);

   PlayoutDelay = media + (4 * varianza);
   ref_PlayoutDelay = PlayoutDelay;
   fprintf(dibujofp, "%d %0.2f\n", secuencia, PlayoutDelay); 
   fputs("1 ", Ofp);    // Escribe un 1 en el fichero (el primer paquete se recibe a tiempo)
   delay = Lee_retraso(&silencio, &secuencia, Ifp, Ofp, dibujofp);

   // se lee el fichero completo
   //feof: devuelve un valor distinto de cero si ha llegado al final del archivo 
   //ferror: devuelve un valor distinto de cero si se ha producido algun error
   while(!feof(Ifp) && !ferror(Ifp) && !ferror(Ofp)){
      nodo = CreaNodo(delay, 0);
      Enlazaspike(&ldelay, nodo);
      Playout_spike(&silencio, ldelay, &PlayoutDelay, &ref_PlayoutDelay, secuencia, &media, &varianza, &var, &mode, dibujofp);
	 
      if (delay < ref_PlayoutDelay){   // el paquete se recibe a tiempo
         fputs("1 ", Ofp);
      }
      else {                       // el paquete no llega a transmitirse
         fputs("0 ", Ofp);
      }
      fprintf(borrar, "%d %f\n", secuencia, PlayoutDelay);
      delay = Lee_retraso(&silencio, &secuencia, Ifp, Ofp, dibujofp);
   }

   Borra_Lista(&ldelay);

   // Se cierran los ficheros
   if(Ifp) { 
      (void)fclose(Ifp);
   }
   if(Ofp) { 
      (void)fclose(Ofp); 
   }
   if(dibujofp) { 
      (void)fclose(Ofp); 
   }      
   return 0 ;
}


 int Playout_spike(float *silencio, NODO *ldelay, float *PlayoutDelay, float
*ref_PlayoutDelay, int secuencia, float *media, float *varianza, float *var,
int *mode, FILE *playoutfp){
      float delay;
      float antdelay;
      float antantdelay;
      float antmedia;
      float antvarianza;
      float antPlayoutDelay;
      int cambio;

var = 0;
cambio = 0;
antmedia = *media;
antvarianza = *varianza;
antPlayoutDelay = *PlayoutDelay;
delay = ldelay->delay;
antdelay = ldelay->sig->delay;
if((ldelay->sig->sig) != NULL){
    antantdelay = ldelay->sig->sig->delay;
}
if(*silencio != 0){
    *ref_PlayoutDelay = *PlayoutDelay;
    if(*ref_PlayoutDelay < antPlayoutDelay - *silencio){
        *ref_PlayoutDelay = antPlayoutDelay - *silencio;
    }
    *silencio = 0;
}
if(*mode == NORMAL){
    if(fabs(delay-antdelay) > ((fabs(antvarianza)*2) + 800)){
  *var = 0;
       *mode = IMPULSE;
    }
}
else{
    *var = (*var/2) + fabs((2*(delay) - antdelay - antantdelay)/8);
    if(*var <= 63){
  *mode = NORMAL;
       cambio = 1;
    }
}
if(cambio == 0){
    if(*mode == NORMAL){

    *media = 0.125 * delay + 0.875 * antmedia;
      }
      else{
    *media = antmedia + delay - antdelay;
      }
      *varianza = 0.125 * fabs(delay - *media) + 0.875 * antvarianza;
  }
  cambio = 0;
  *PlayoutDelay = *media + (BETA * (*varianza));
  fprintf(playoutfp, "%d %0.4f\n", secuencia, *ref_PlayoutDelay);
  return 0;
}

