/************************************************************************/
/* Fichero: tipo.c                                                      */
/* Descripcon: funcion que lee el fichero codificado por el codec g723 */
/* o g729 y escribe en un fichero de salida el tipo de trama que se ha  */ 
/* generado: 1 si activa, 2 si SID y 0 si no transmitida.               */
/* Llamada: tipo codficador fich_codif fich_tipo                        */
/************************************************************************/

#include <stdio.h>

#include "typedef.h"

#define RATE_8000       80      /* Full rate  (8000 bit/s)       */
#define SERIAL_SIZE (80+2)      /* bfi+ number of speech bits    */
#define RATE_SID_OCTET    16    /* number of bits in Octet Transmission mode */

int read_frame_g723(FILE *f_serial, FILE *f_type );
int read_frame_g729(FILE *f_serial, FILE *f_type );

int main( int argc, char *argv[] ){

   FILE    *f_serial, *f_type ;            /* I/O File pointers */
   Word32  count_frame = 0L ;

   if (argc != 4 ) {
      printf("Uso: %s codificador fich_codificado fich_tipo\n", argv[0]);
      printf("Codificador: g723 o g729\n");
      exit(1);
   }

   f_serial = fopen( argv[2], "rb") ;  
   if ( f_serial == NULL ) {
      fprintf(stderr, "Fichero de entrada no valido: %s\n", argv[2] ) ;
      exit(1) ;
    }

   f_type = fopen( argv[3], "w") ;
   if ( f_type == NULL ) {
      fprintf(stderr, "Fichero de salida no valido: %s\n", argv[3] ) ;
      exit(1) ;
   }

   if(strcmp(argv[1],"g723") == 0){
      while(read_frame_g723(f_serial, f_type) != -1){
         printf("Frame = %ld\r", count_frame++);
      }
   }
   else if(strcmp(argv[1],"g729") == 0){
      while(read_frame_g729(f_serial, f_type) != -1){
        count_frame++;
	//printf("Frame = %ld\r", count_frame++);
      }
   }
   else{
      fprintf(stderr, "Codificador no valido. Opciones: g723 o g729\n");
      exit(1);
   }

   //printf("%ld tramas leidas\n", count_frame);

   if(f_serial) { (void)fclose(f_serial); }
   if(f_type) { (void)fclose(f_type); }

   return 0;
}

/* Funcion read_frame_g723 */
/* Lee del fichero de entrada las tramas codificadas. Primero lee la     */
/* cabecera para determinar el tipo de trama y el numero de bytes a leer */
/* A continuacion lee los datos de la trama (si no es no transmitida)    */
/* Escribe el tipo de trama en el fichero indicado: 1 activa, 2 sid,     */
/* y 0 no tranmitida                                                     */

int read_frame_g723( FILE *f_serial, FILE *f_type )
{
    Word16  Info ;
    int     Size ;
    char    Line[24] ;

    if(fread( Line, 1,1, f_serial ) != 1) return(-1);

    Info = Line[0] & (Word16)0x0003 ;

    /* Check frame type and rate informations */
    switch(Info) {

        /* Active frame, high rate */
        case 0 : {
            Size  = 23;
	    fprintf(f_type, "1 ");
            break;
        }

        /* Active frame, low rate */
        case 1 : {
            Size  = 19;
	    fprintf(f_type, "1 ");
            break;
        }

        /* Sid Frame */
        case 2 : {
            Size  = 3;
            fprintf(f_type, "2 ");
            break;
        }

        /* untransmitted */
        default : {
            fprintf(f_type, "0 ");
        }
    }
    if(Info == 0 || Info == 1 || Info ==2){
       fread( &Line[1], Size , 1, f_serial ) ;
    }
    return(0);
}

/* Funcion read_frame_g729 */
/* Lee del fichero de entrada las tramas codificadas. Primero lee la     */
/* cabecera para determinar el numero de bytes a leer y a continuacion   */
/* lee los datos de la trama. El tamano de los datos determina el tipo   */
/* de trama, que se escribe en el fichero indicado: 1 activa, 2 sid, y 0 */ 
/* no transmitida                                                        */

int read_frame_g729(FILE *f_serial, FILE *f_type){

  Word16  serial[SERIAL_SIZE];    /* Serial stream               */
  Word16 nb_bits;

  if(fread(serial, sizeof(short), 2, f_serial) != 2) {
    return(-1);
  }

  if(fread(&serial[2], sizeof(Word16), (size_t)serial[1], f_serial)
     != (size_t)serial[1]) {   //Se lee la trama codificada
    return(-1);
  }

  nb_bits = serial[1]++;  //Se ve el tamano de la trama

  if(nb_bits == RATE_8000) {           // Trama activa
     fprintf(f_type, "1 ");
  }
  else if(nb_bits == RATE_SID_OCTET) { // Trama SID
     fprintf(f_type, "2 ");
  }
  else {                               // Trama no tranmsitida
     fprintf(f_type, "0 ");
  }
  return(0);
}


